﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Concurrent;
using System.Threading.Tasks;

namespace AlpacaProject.ArchitecturalPatterns
{
    public abstract class ConsumerBase<TItem>
    {

        private BlockingCollection<TItem> _buffer;

        public ConsumerBase(BlockingCollection<TItem> buffer)
        {
            _buffer = buffer;

            Name = this.GetType().Name;
        }

        public string Name { get; set; }

        public void Run()
        {
            foreach (TItem item in _buffer.GetConsumingEnumerable())
                Consume(item);
        }

        protected abstract void Consume(TItem item);

    }

    public class Consumer<TItem> : ConsumerBase<TItem>
    {

        private Action<TItem> _body;

        public Consumer(BlockingCollection<TItem> buffer, Action<TItem> body)
            : base(buffer)
        {
            if (body == null) throw new ArgumentNullException("body");

            _body = body;
        }

        protected override void Consume(TItem item)
        {
            _body(item);
        }

    }

    public class GenericItemConsumer<TItem> : ConsumerBase<TItem>
    {

        public GenericItemConsumer(BlockingCollection<TItem> buffer)
            : base(buffer)
        {
        }

        protected override void Consume(TItem item)
        {
            Console.WriteLine("{0} consumed item {1}", Name, item);
        }

    }
}
