﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Concurrent;
using System.Threading.Tasks;
using System.Threading;

namespace AlpacaProject.ArchitecturalPatterns
{
    public abstract class ProducerBase<TItem>
    {

        private BlockingCollection<TItem> _buffer;

        public ProducerBase(BlockingCollection<TItem> buffer)
        {
            _buffer = buffer;

            Name = this.GetType().Name;
        }

        public string Name { get; set; }

        public void Run()
        {
            while (!Done())
            {
                TItem item = Produce();
                _buffer.Add(item);
            }
        }

        /// <exception cref="OperationCanceledExcepiton">If the <see cref="CancellationToken"/> is canceled.</exception>
        public void Run(CancellationToken cancellationToken)
        {
            while (!Done())
            {
                TItem item = Produce();
                _buffer.Add(item, cancellationToken);
            }
        }

        protected abstract TItem Produce();

        protected abstract bool Done();

    }

    /// <summary>
    /// A producer that just instantiates a fixed number of items.
    /// </summary>
    /// <typeparam name="TItem"></typeparam>
    public class NewItemProducer<TItem> : ProducerBase<TItem>
        where TItem : class, new()
    {

        private int _countToProduce;
        private int _producedCount;

        public NewItemProducer(BlockingCollection<TItem> buffer, int countToProduce)
            : base(buffer)
        {
            _countToProduce = countToProduce;
        }

        protected override TItem Produce()
        {
            _producedCount++;
            var item = new TItem();
            Console.WriteLine("{0} produced item {1}", Name, item);
            return item;
        }

        protected override bool Done()
        {
            return _producedCount >= _countToProduce;
        }
    }
}
