﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;
using System.Threading.Tasks;
using AntisocialRobots;
using System.Threading;

namespace AlpacaProject
{
    public class DataRaceExamples
    {

        [UnitTestMethod]
        [DataRaceTestMethod]
        public void DataRace_ReadWrite()
        {
            int x = 1;
            Parallel.Invoke(
                () => x = 2
                , () => Console.WriteLine("x={0}", x)
                );
            // May output 1 or 2 depending on schedules.

            Assert.IsTrue(x == 2);
        }

        [UnitTestMethod]
        [DataRaceTestMethod]
        public void DataRace_TwoWrites()
        {
            int x = 1;
            Parallel.Invoke(
                () => x = 2
                , () => x = 3
                );

            Console.WriteLine("x={0}", x);

            // May output 1 or 2 depending on the schedule of the test.
            Assert.IsTrue(x == 2 || x == 3);
        }

        [DataRaceTestMethod]
        public void Quiz_WhereAreTheDataRaces1()
        {
            int[] a = new int[2];
            int x;

            Parallel.For(1, 2, i => {
                x = a[i];
            });
        }

        [DataRaceTestMethod]
        public void Quiz_WhereAreTheDataRaces2()
        {
            int[] a = new int[2];
            int x = 2010;

            Parallel.For(1, 2, i => {
                a[i] = x;
            });
        }

        [DataRaceTestMethod]
        public void Quiz_WhereAreTheDataRaces3()
        {
            int[] a = new int[3];

            Parallel.For(1, 2, i => {
                a[i] = a[i + 1];
            });
        }

        [DataRaceTestMethod]
        public void LockExample1()
        {
            int x = 0, w = 0, z = 0, y = 0;

            Object myLock = new object(); // use to protect x

            Parallel.Invoke(
                () => { lock (myLock)x = 2; }
                , () => w = 2
                , () => z = 2
                , () => { lock (myLock) y = x; }
                );

            Assert.AreEqual(2, w);
            Assert.AreEqual(2, z);
            Assert.AreEqual(2, x);
            Assert.IsTrue(y == 0 || y == 2);    // Due to race, we won't know which one it'll actually be.
        }

        private TaskMeter task1 = new TaskMeter("Task1");
        private TaskMeter task2 = new TaskMeter("Task2");

        [PerformanceTestMethod(Repetitions = 10)]
        public void GlobalLock_DestroysParallelism()
        {
            // Note: This example isn't showing any data races, just locking scenarios
            object gameboard = new object();
            var r1 = new Robot();
            var r2 = new Robot();

            Parallel.Invoke(
                () => {
                    lock (gameboard)
                    {
                        task1.Start();
                        MoveRobot(r1);
                        task1.End();
                    }
                }
                , () => {
                    lock (gameboard)
                    {
                        task2.Start();
                        MoveRobot(r2);
                        task2.End();
                    }
                }
                );
        }

        [PerformanceTestMethod(Repetitions = 10)]
        public void FineGrainedLock_KeepsParallelism()
        {
            // Note: This example isn't showing any data races, just locking scenarios
            object r1Lock = new object();
            object r2Lock = new object();
            var r1 = new Robot();
            var r2 = new Robot();

            Parallel.Invoke(
                () => {
                    lock (r1Lock)
                    {
                        task1.Start();
                        MoveRobot(r1);
                        task1.End();
                    }
                }
                , () => {
                    lock (r2Lock)
                    {
                        task2.Start();
                        MoveRobot(r2);
                        task2.End();
                    }
                }
                );
        }

        private void MoveRobot(Robot r1)
        {
            // Doing some stuff...
            Thread.Sleep(20);
        }

    }
}
