﻿using AntisocialRobots;
using Microsoft.Concurrency.TestTools.UnitTesting;
using System;
using System.Threading.Tasks;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using Microsoft.Concurrency.TestTools.UnitTesting.Chess;

namespace AlpacaProject
{
    /// <summary>
    ///</summary>
    public class RobotSimulationScenario_SwapPlaces
    {

        static int[] MyRoomX = new[] {
                0, 1,      4, 5,
                0, 1,      4, 5,
                0, 1,      4, 5,
            };

        static int[] MyRoomY = new[] {
                0, 0,      0, 0,
                1, 1,      1, 1,
                2, 2,      2, 2,
            };

        public static readonly RoomPoint R1_StartLocation = new RoomPoint(2, 1);
        public static readonly RoomPoint R2_StartLocation = new RoomPoint(3, 1);

        public static RobotSimulation CreateSimulation(out Robot r1, out Robot r2)
        {
            var simulation = RobotSimulationScenarioUtil.RobotSimulationFactory(6, MyRoomX, MyRoomY, false);

            r1 = simulation.CreateRobot(R1_StartLocation);
            r2 = simulation.CreateRobot(R2_StartLocation);

            return simulation;
        }

        /// <summary>
        /// Verifies the DataRaceScenario map setup is correct.
        /// </summary>
        [UnitTestMethod]
        public void TestScenarioBehavior()
        {
            var roomA = RobotSimulationScenarioUtil.RobotSimulationFactory(6, MyRoomX, MyRoomY, false);
            var r1 = roomA.CreateRobot(R1_StartLocation);
            roomA.PerformFrameUpdate();
            //Console.WriteLine("r1.Location={0}", r1.Location);
            Assert.AreEqual(R2_StartLocation, r1.Location, "Robot 1: Doesn't want to move to the location we expect for this test.");

            var roomB = RobotSimulationScenarioUtil.RobotSimulationFactory(6, MyRoomX, MyRoomY, false);
            var r2 = roomB.CreateRobot(R2_StartLocation);
            roomB.PerformFrameUpdate();
            //Console.WriteLine("r2.Location={0}", r2.Location);
            Assert.AreEqual(R1_StartLocation, r2.Location, "Robot 2: Doesn't want to move to the location we expect for this test.");

            // And test the full simulation
            var roomC = CreateSimulation(out r1, out r2);
            roomC.PerformFrameUpdate();
            //Console.WriteLine("r1.Location={0}", r1.Location);
            //Console.WriteLine("r2.Location={0}", r2.Location);
            Assert.AreEqual(R1_StartLocation, r1.Location, "Robot1: When both placed in the room, both robots shouldn't be able to move since their desired location is taken up by the other robot.");
            Assert.AreEqual(R2_StartLocation, r2.Location, "Robot2: When both placed in the room, both robots shouldn't be able to move since their desired location is taken up by the other robot.");
        }

    }
}
