﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AlpacaProject.ReplicationPatterns
{
    class DoubleBufferedGame
    {

        GameObject[,] _objects;
        int _framesToRun;
        int _frameIdx;

        public DoubleBufferedGame(int numObjects)
        {
            _objects = new GameObject[2, numObjects];
        }

        public void Run(int framesToRun)
        {
            // Initialize all the object instances
            int numObjects = _objects.GetLength(1);
            for (int bufIdx = 0; bufIdx < 2; bufIdx++)
                for (int i = 0; i < numObjects; i++)
                    _objects[bufIdx, i] = new GameObject();

            _framesToRun = framesToRun;
            _frameIdx = 0;

            GameLoop();
        }

        private bool Done() { return _frameIdx < _framesToRun; }

        private void GameLoop()
        {
            int currentBufferIdx = 0;
            while (!Done())
            {
                Parallel.Invoke(
                    () => DrawAllCells(currentBufferIdx)
                    , () => UpdateAllCells(currentBufferIdx)
                    );

                currentBufferIdx = 1 - currentBufferIdx;
                _frameIdx++;
            }
        }

        void DrawAllCells(int bufferIdx)
        {
            foreach (var g in _objects)
                g.Draw();
        }

        void UpdateAllCells(int bufferIdx)
        {
            int numObjects = _objects.GetLength(1);
            int writeBufferIdx = 1 - bufferIdx;
            for (int i = 0; i < numObjects; i++)
                _objects[writeBufferIdx, i] = _objects[bufferIdx, i].ImmutableUpdate();
        }

    }
}
