﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AlpacaProject.ReplicationPatterns
{
    class ImmutableGame
    {

        GameObject[] _objects;
        int _framesToRun;
        int _frameIdx;

        public ImmutableGame(int numObjects)
        {
            _objects = new GameObject[numObjects];
        }

        public void Run(int framesToRun)
        {
            // Initialize all the object instances
            for (int i = 0; i < _objects.Length; i++)
                _objects[i] = new GameObject();

            _framesToRun = framesToRun;
            _frameIdx = 0;

            GameLoop();
        }

        private bool Done() { return _frameIdx < _framesToRun; }

        private void GameLoop()
        {
            while (!Done())
            {
                GameObject[] newObjects = null;

                Parallel.Invoke(
                    DrawAllCells
                    , () => newObjects = UpdateAllCells()
                    );

                _objects = newObjects;
                _frameIdx++;
            }
        }

        void DrawAllCells()
        {
            foreach (var g in _objects)
                g.Draw();
        }

        GameObject[] UpdateAllCells()
        {
            GameObject[] newObjects = new GameObject[_objects.Length];
            for (int i = 0; i < _objects.Length; i++)
                newObjects[i] = _objects[i].ImmutableUpdate();
            return newObjects;
        }

    }
}
