﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;

namespace AlpacaProject.ReplicationPatterns
{
    public class SimpleImmutableListTests
    {

        class MyValue
        {
            public int Value;

            public MyValue() { }
            public MyValue(int v) { Value = v; }
        }

        [UnitTestMethod]
        public void CreateNew()
        {
            var list = new SimpleImmutableList<MyValue>();
            Assert.AreEqual(0, list.Count);
            Assert.Throws<IndexOutOfRangeException>(() => { MyValue v = list[0]; });
        }

        [UnitTestMethod]
        public void Add1()
        {
            var list = new SimpleImmutableList<MyValue>();

            var v1 = new MyValue(1);
            var newList = list.Add(v1);

            // Make sure the empty list still behaves the same
            Assert.AreEqual(0, list.Count);
            Assert.Throws<IndexOutOfRangeException>(() => { MyValue v = list[0]; });

            // And the new list
            Assert.AreEqual(1, newList.Count);
            Assert.AreSame(v1, newList[0]);
        }

        [UnitTestMethod]
        public void Add2()
        {
            var list = new SimpleImmutableList<MyValue>();

            var v1 = new MyValue(1);
            var newList = list.Add(v1);

            var v2 = new MyValue(2);
            var newList2 = newList.Add(v2);

            // Make sure the empty list still behaves the same
            Assert.AreEqual(0, list.Count);
            Assert.Throws<IndexOutOfRangeException>(() => { MyValue v = list[0]; });

            // And the new lists
            Assert.AreEqual(2, newList2.Count);
            Assert.AreSame(v1, newList[0]);
            Assert.Throws<IndexOutOfRangeException>(() => { MyValue v = newList[1]; });

            Assert.AreSame(v1, newList2[0]);
            Assert.AreSame(v2, newList2[1]);
        }

    }
}
