﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Concurrency.TestTools.UnitTesting;

namespace AlpacaProject
{

    /// <summary>Counter that is NOT thread-safe because it doesn't protect it's internal value.</summary>
    class UnsafeCounter
    {
        private int value;

        public UnsafeCounter()
        {
            value = 0;
        }

        public int Count { get { return value; } }

        public void Increment()
        {
            value++;
        }

    }

    /// <summary>Thread-safe counter.</summary>
    class SafeCounter
    {
        private int value;
        private object syncObj = new object();

        public SafeCounter()
        {
            value = 0;
        }

        public int Count { get { lock (syncObj) return value; } }

        public void Increment()
        {
            lock (syncObj)
                value++;
        }

    }

    public class SafeCounterTests
    {

        [UnitTestMethod]
        [DataRaceTestMethod]
        public void UnsafeCounterTest()
        {
            UnsafeCounter counter = new UnsafeCounter();

            Parallel.For(0, 10, i => counter.Increment());

            Assert.AreEqual(10, counter.Count);
        }

        [DataRaceTestMethod]
        public void SafeCounterTest()
        {
            SafeCounter counter = new SafeCounter();

            Parallel.For(0, 10, i => counter.Increment());

            Assert.AreEqual(10, counter.Count);
        }

    }
}
